package contrabase;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

/**
 * @author Jong10
 * Contrabase 의 설정에 접근하는 static 클래스
 */
public class ContrabaseProperties {
	private String filename = "contrabase.properties";
	private Properties properties = null;

	/**
	 * key 에 해당하는 프로퍼티를 가져온다.
	 * 
	 * @param key 가져올 키
	 * @return 해당 key 에 해당하는 프로퍼티
	 */
	public String get(String key) {
		return properties.getProperty(key);
	}

	public String getFilename() {
		return filename;
	}

	/**
	 * @return 파일이름
	 */
	private static String getPropertiesFilename(String filename) {
		return System.getProperty("user.dir") + File.separator + "conf" + File.separator + filename;
	}

	/**
	 * 설정파일을 읽는다.
	 * @throws IOException 
	 */
	public ContrabaseProperties(String filename) throws ContrabaseException {
		if (properties == null) {
			properties = new Properties();
			FileInputStream fis = null;

			try {
				fis = new FileInputStream(getPropertiesFilename(filename));
				properties.load(fis);
			} catch (FileNotFoundException e) {
				System.err.println("File not found : " + filename);
				throw new ContrabaseException(e);
			} catch (IOException e) {
				System.err.println(e.getMessage());
				throw new ContrabaseException(e);
			} finally {
				if (fis != null) {
					try {
						fis.close();
					}
					catch(IOException ex) {
					}
				}
			} // end of try
			
		}
	}
}
