package contrabase.api;

import java.io.IOException;
import java.io.Serializable;

import contrabase.util.Converter;

public class UpdateValue implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private String fieldName = null;
	private String valueString = null;
	private byte[] valueBytes = null;

	private boolean decimal = false;	// 정수
	private boolean text = false;		// char, String
	private boolean bool = false;		// 참/거짓
	private boolean number = false;		// 실수 + 정수
	private boolean blob = false;		// BLOB

	public String getValueString() {
		return valueString;
	}

	public String getFormattedValueString() {
		if (text)
			return "'" + valueString + "'";
		else
			return valueString;
	}

	public byte[] getValueBytes() {
		return valueBytes;
	}

	public String getFieldName() {
		return "'" + fieldName + "'";
	}

	public boolean isDecimal() {
		return decimal;
	}

	public boolean isText() {
		return text;
	}

	public boolean isBool() {
		return bool;
	}

	public boolean isNumber() {
		return number;
	}

	public boolean isBlob() {
		return blob;
	}

	// DECIMAL
	public UpdateValue(String fieldName, Byte value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.decimal = true;
		this.number = true;
	}

	public UpdateValue(String fieldName, Short value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.decimal = true;
		this.number = true;
	}

	public UpdateValue(String fieldName, Integer value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.decimal = true;
		this.number = true;
	}

	public UpdateValue(String fieldName, Long value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.decimal = true;
		this.number = true;
	}

	// REAL
	public UpdateValue(String fieldName, Float value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.number = true;
	}

	public UpdateValue(String fieldName, Double value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.number = true;
	}

	// TEXT
	public UpdateValue(String fieldName, Character value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.text = true;
	}

	public UpdateValue(String fieldName, String value) {
		this.fieldName = fieldName;
		this.valueString = value.toString();
		this.text = true;
	}

	// BOOLEAN
	public UpdateValue(String fieldName, Boolean value) {
		this.fieldName = fieldName;
		if (value == true)
			this.valueString = "1";
		else
			this.valueString = "0";
		this.bool = true;
	}

	// BLOB
	public UpdateValue(String fieldName, Serializable value) throws IOException {
		this.fieldName = fieldName;
		this.valueBytes = Converter.objectToBytes(value);
		this.blob = true;
	}
}
