package contrabase.condition;

import java.util.SortedSet;

final public class Term implements Condition {
	private static final long serialVersionUID = 1L;

	public enum TYPE { NUMBER, TEXT, BOOLEAN };

	private String field;
	private String condition;
	private String value;
	private TYPE type;

	Term(String field, String condition, String value, TYPE species) {
		// XXX: 여기서, condition 을 검사해야 하나? 아니면, 다른데서??
		this.field = field;
		this.condition = condition;
		this.value = value;
		this.type = species;
	}

	@Override
	public SortedSet<String> getKeys(KeySelector keySelector) {
		return keySelector.getKeysByTerm(this);
	}

	public String toString() {
		return new StringBuilder()
					.append("(")
					.append(field)
					.append(" ")
					.append(condition)
					.append(" ")
					.append(getFormattedValue())
					.append(")")
					.toString();
	}

	/**
	 * 필드 이름을 반환한다.
	 * @return
	 */
	public String getField() {
		return field;
	}

	/**
	 * 비교 기호를 반환한다.
	 * @return
	 */
	public String getCondition() {
		return condition;
	}

	/**
	 * value 를 String 으로 돌려준다.
	 * @return 작은 따옴표 처리가 되지 않은 문자열
	 */
	public String getValue() {
		return value;
	}

	/**
	 * 글자 타입이면, value 에 작은 따옴표를 감싸고, 숫자면 그대로.
	 * @return 작은 따옴표가 처리된 value 에 대한 문자열.
	 */
	public String getFormattedValue() {
		if (type.compareTo(TYPE.TEXT) == 0)
			return "'" + value + "'";
		else
			return value;
	}

	/**
	 * TYPE 형식으로 종류를 돌려준다.
	 * @return
	 */
	public TYPE getType() {
		return type;
	}

	/**
	 * 해당 문장이, key인지 확인하는 내용인지 알려준다.
	 * @return
	 */
	public boolean isKeyCompare() {
		return "<key>".equals(field) && "==".equals(condition) && !("*".equals(value));
	}
	
	/**
	 * isKeyCompare() 기능과 비슷하나 아래 메소드는 전체 Key List를 확인하는 내용인지 알려준다.
	 * @return
	 */
	public boolean isAllKeyCompare() {
		return "<key>".equals(field) && "==".equals(condition) && "*".equals(value);
	}
}
