package contrabase.master;

import contrabase.ContrabaseException;
import contrabase.ContrabaseProperties;
import contrabase.PROPERTY;
import contrabase.network.ContrabaseNetworkException;
import contrabase.network.MessageHandler;
import contrabase.network.ServerSideThread;
import contrabase.repository.datastore.ContrabaseDatastoreException;

public class MasterMain {
	ContrabaseProperties properties;

	private MessageHandler repositoryHandler;
	private int repositoryHandlerPort;

	private MessageHandler serviceHandler;
	private int serviceHandlerPort;

	private String dataDir;
	private String logDir;
	
	private MasterGUI masterGUI;


	/**
	 * 마스터부분의 메인부분.
	 * 서버 사이드 스레드 시작
	 * 
	 * @param args
	 * @throws ContrabaseDatastoreException 
	 */
	public static void main(String args[]) {
		MasterMain d = new MasterMain("contrabase.properties");
		try {
			d.startService();
		} catch (ContrabaseDatastoreException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Dispatcher 클래스의 생성자
	 * @param manager
	 */
	public MasterMain(String propertiesFilename) {
		RepositoriesManager manager = null;
		try {
			properties = new ContrabaseProperties(propertiesFilename);
			this.dataDir = properties.get("data");
			manager = new RepositoriesManager(dataDir);
		} catch (ContrabaseException e) {
			e.printStackTrace();
			System.err.println("Cannot open properties file: " + properties.getFilename());
			System.exit(1);		// 설정파일을 읽을 수 없으면, 종료한다.
		}

		// Log를 보여주는 Master GUI를 시작한다. 
		this.logDir = properties.get("log");
		masterGUI = new MasterGUI(logDir);
		masterGUI.start();

		// repositoryHandler = new RepositoryHandler(manager);
		repositoryHandler = new RepositoryHandler(manager, masterGUI);
		repositoryHandlerPort = Integer.parseInt(properties.get(PROPERTY.MASTER_FROM_REPOSITORY_PORT));
		
		// serviceHandler = new ServiceHandler(manager, dataDir);
		serviceHandler = new ServiceHandler(manager, dataDir, masterGUI);
		serviceHandlerPort = Integer.parseInt(properties.get(PROPERTY.MASTER_FROM_CLIENT_PORT));
	}

	/**
	 * 마스터의 서비스를 시작한다.
	 * @throws ContrabaseDatastoreException 
	 */
	public void startService() throws ContrabaseDatastoreException {
		try {
			MasterTable.createMetaTable(dataDir);
			
			// 리포지토리와 연결을 해주는 쓰레드를 시작한다.
			ServerSideThread repositoryHandlerThread = new ServerSideThread(repositoryHandlerPort, repositoryHandler);
			repositoryHandlerThread.start();
			
			// 클라이언트의 요청을 받는 쓰레드를 시작한다.
			ServerSideThread clientServiceThread = new ServerSideThread(serviceHandlerPort, serviceHandler);
			clientServiceThread.start();
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
}
