package contrabase.master;


import java.sql.*;

import contrabase.repository.datastore.ContrabaseDatastoreException;
import contrabase.util.FileUtil;

/**
 * userInform Table 과 masterSection Table 을 생성하는 클래스 
 * userInform(
 * 		userId TEXT PRIMARY KEY NOT NULL, 
 * 		password TEXT)
 * 
 *  masterSection(
			 userId TEXT NOT NULL,
			 tableName TEXT,
			 repositoryId TEXT,
			 endKey TEXT,
			 primary key(userId, tableName, repositoryId))
 */
public class MasterTable {
	public static void createMetaTable(String dataDir) throws ContrabaseDatastoreException {
		Connection conn = null;
		FileUtil.createDirectory(dataDir);
		String url = "jdbc:sqlite:" + dataDir + "/master.db";
		PreparedStatement pstat = null;
		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection(url);
			
			String sqlUserInform = "create table if not exists userInform(userId TEXT PRIMARY KEY NOT NULL, password TEXT)";
			String sqlMasterSection = "create table if not exists masterSection("
					+ "userId TEXT NOT NULL,"
					+ "tableName TEXT,"
					+ "repositoryId TEXT,"
					+ "endKey TEXT,"
					+ "primary key(userId, tableName, repositoryId))";
			pstat = conn.prepareStatement(sqlUserInform);
			pstat.execute();
			
			pstat = conn.prepareStatement(sqlMasterSection);
			pstat.execute();
	
		} catch (SQLException e) {
			throw new ContrabaseDatastoreException("SQLException occur", e);
		} catch (ClassNotFoundException e) {
			throw new ContrabaseDatastoreException("org.sqlite.JDBC is not found", e);
		} finally {
			try {
				if (pstat != null)
					pstat.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}

}
