package contrabase.master;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import contrabase.network.ClientSide;


/**
 * @author Rivers
 * 해당 Repository 의 정보를 관리하는 Manager Class 
 * Map으로 Repository의 정보를 관리한다. 
 */
public class RepositoriesManager {
	private HashMap<String, RepositoryEntry> repositoryEntries = new HashMap<String, RepositoryEntry>();
	private Lock systemLock = new Lock();
	private String dataDir;

	public RepositoriesManager(String dataDir) {
		this.dataDir = dataDir;
	}
	
	public RepositoryEntry getRepositoryEntry(String rid) {
		return repositoryEntries.get(rid);
	}

	// 값들을 리턴한다.
	public Collection<RepositoryEntry> values() {
		return repositoryEntries.values();
	}

	// XXX: 이미 있는 rid 에 해당하는 리포지토리를 넣으면, 덮어씌워지는게 맞나??
	public void setRepositoryEntry(RepositoryEntry entry) {
		repositoryEntries.put(entry.getRid(), entry);
	}

	public Iterator<RepositoryEntry> iterator() {
		return repositoryEntries.values().iterator();
	}

	public boolean hasRepositoryEntry(String rid) {
		return repositoryEntries.containsKey(rid);
	}

	/**
	 * MasterSection Table을 검색하여 
	 * userId, tableName 에서, 해당 key 의 범위를 가진 리포지토리를 찾아서,
	 * rid를 리턴한다.
	 * 
	 * @param userId 사용자 아이디
	 * @param tableName
	 * @param key
	 * @return rid
	 */
	public String getRidByKey(String userId, String tableName, String key) {
		Map<String, String> map = new HashMap<String, String>();
		Connection conn = null;
		String url = "jdbc:sqlite:" + dataDir + "/master.db";
		String sql = "select repositoryId, endKey from masterSection " +
				"where userId = ? and tableName = ? ";
		
		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection(url);
			PreparedStatement pstat = conn.prepareStatement(sql);
			
			pstat.setString(1, userId);
			pstat.setString(2, tableName);
			ResultSet rs = pstat.executeQuery();
			
			while (rs.next()) {
				String endKey = rs.getString("endKey");
				String repositoryId = rs.getString("RepositoryId");
				map.put(endKey, repositoryId);					 
			}
			
			String[] endKeyList = map.keySet().toArray(new String[0]);
			Arrays.sort(endKeyList);
			
			for (int i = 0; i < endKeyList.length; i++) {
				if (key.compareTo(endKeyList[i]) <= 0) {
					return map.get(endKeyList[i]);
				}
			}	
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return null;
	}

	////////////////////////////////////////////////////////////
	// 로드 밸런싱과 관련된 락

	public synchronized void waitForLoadBalancing() throws InterruptedException {
		systemLock.waitForThis();
	}

	public synchronized void startLoadBalancing() throws InterruptedException {
		systemLock.lock();
	}

	public synchronized void endLoadBalancing() throws InterruptedException {
		systemLock.unlock();
	}

	public int getSize() {
		return repositoryEntries.size();
	}

	public ClientSide[] getConnections() {
		List<ClientSide> connections = new ArrayList<ClientSide>();
		
		for (RepositoryEntry rep : repositoryEntries.values())
			connections.add(rep.getClientSide());
		
		return connections.toArray(new ClientSide[0]);
	}
}
