package contrabase.master.actionhandler;

import contrabase.master.MasterSectionDAO;
import contrabase.master.RepositoriesManager;
import contrabase.network.MSG;
import contrabase.network.MapReduce;
import contrabase.network.Message;
import contrabase.repository.datastore.ContrabaseDatastoreException;

/**
 * @author jong10
 * 테이블을 삭제하는 액션
 */
public class ActionDeleteTable extends ActionHandler {

	private String dataDir;

	public ActionDeleteTable(RepositoriesManager manager, String dataDir) {
		super(manager);
		this.dataDir = dataDir;
	}

	@Override
	public void action(Message request, Message response) {
		String userId = (String)request.get(MSG.REQUEST.USER_ID);
		String tableName = (String)request.get(MSG.REQUEST.TABLE_NAME);
		
		MasterSectionDAO section = null;
		
		try {
			section = new MasterSectionDAO(dataDir, userId, tableName);
			
			// 해당 테이블을 실제로 지운다.
			if (section.deleteAll()) {
				// 테이블 삭제 요청 메시지를 만든다.
				Message msg = new Message();
				msg.set(MSG.ACTION, MSG.REQUEST.DELETE_TABLE);
				msg.set(MSG.REQUEST.USER_ID, userId);
				msg.set(MSG.REQUEST.TABLE_NAME, tableName);
				
				// Repository 들에게, 태블릿을 지우라고 요청을 뿌린다.
				MapReduce mapReduce = new MapReduce(manager.getConnections());
				Message[] responses = null;
				
				try {
					responses = mapReduce.map(msg);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				
				// 에러가 있었는지 확인한다.
				if (hasFailed(responses))
					throw new RuntimeException("failed: delete table");
				
				// 응답을 성공으로 체크한다.
				response.setSuccessed();
			}
			else {
				response.setFailed("not exist table: " + tableName);
			}
		} catch (ContrabaseDatastoreException e) {
			e.printStackTrace();
			response.setFailed(e.getMessage());
		} finally {
			try {
				if (section != null)
					section.close();
			} catch (ContrabaseDatastoreException e) {
				e.printStackTrace();
			}
		}
	}
}
