package contrabase.master.actionhandler;

import contrabase.master.MasterSectionDAO;
import contrabase.master.RepositoriesManager;
import contrabase.network.MSG;
import contrabase.network.Message;
import contrabase.repository.datastore.ContrabaseDatastoreException;

/**
 * @author jong10
 * 서버에서 테이블이 있는지 확인하는 액션
 */
public class ActionGetTable extends ActionHandler {
	private String dataDir;

	public ActionGetTable(RepositoriesManager manager, String dataDir) {
		super(manager);
		this.dataDir = dataDir;
	}

	@Override
	public void action(Message request, Message response) {
		String userId = (String)request.get(MSG.REQUEST.USER_ID);
		String tableName = (String)request.get(MSG.REQUEST.TABLE_NAME);
		
		MasterSectionDAO section = null;
		
		try {
			section = new MasterSectionDAO(dataDir, userId, tableName);
			
			// 해당 <유저이름, 테이블이름>의 조합이 이미 존재하는지 확인한다.
			if (section.isExist())
				response.setSuccessed();
			else
				response.setFailed("not exist table: " + tableName);
		} catch (ContrabaseDatastoreException e) {
			e.printStackTrace();
			response.setFailed(e.getMessage());
		} finally {
			try {
				if (section != null)
					section.close();
			} catch (ContrabaseDatastoreException e) {
				e.printStackTrace();
			}
		}
	}
}
