package contrabase.master.actionhandler;

import contrabase.ObjectContainer;
import contrabase.master.RepositoriesManager;
import contrabase.master.RepositoryEntry;
import contrabase.network.ClientSide;
import contrabase.network.ContrabaseNetworkException;
import contrabase.network.MSG;
import contrabase.network.Message;
import contrabase.repository.datastore.TabletEntry;

public class ActionInsert extends ActionHandler {

	public ActionInsert(RepositoriesManager manager) {
		super(manager);
	}

	@Override
	public void action(Message request, Message response) {
		String userId = (String)request.get(MSG.REQUEST.USER_ID);
		String tableName = (String)request.get(MSG.REQUEST.TABLE_NAME);
		
		ObjectContainer oc = (ObjectContainer)request.get(MSG.REQUEST.INSERT_DATA);
		String key = oc.getKey();
		
		// 해당 key 를 가지고 있는 리포지토리를 찾는다.
		String rid = manager.getRidByKey(userId, tableName, key);
		RepositoryEntry repository = manager.getRepositoryEntry(rid);
		ClientSide clientSide = repository.getClientSide();
		
		// 해당 리포지토리에게 Insert 메시지를 그대로! 보내고,
		Message repositoryResponse = null;
		try {
			repositoryResponse = clientSide.send(request);
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
			try {
				clientSide.close();
			} catch (ContrabaseNetworkException ex) {
				ex.printStackTrace();
			}
			
			// 에러나면, 액션을 종료.
			response.setFailed("network error");
			return;
		}
		
		// API 에게 돌려줄 response 를 채워준다.
		if (repositoryResponse.isSuccessed()) {
			response.setSuccessed();
			
			// 해당 엔트리에 count 를 1 증가시켜 준다.
			TabletEntry entry = repository.getTabletEntries().get(userId, tableName);
			entry.increaseCount(1);
		}
		else
			response.setFailed(repositoryResponse.getErrorMessage());
	}
}
