package contrabase.master.actionhandler;

import contrabase.master.RepositoriesManager;
import contrabase.network.MSG;
import contrabase.network.MapReduce;
import contrabase.network.Message;

public class ActionUpdate extends ActionHandler {

	public ActionUpdate(RepositoriesManager manager) {
		super(manager);
	}

	@Override
	public void action(Message request, Message response) {
		String userId = (String)request.get(MSG.REQUEST.USER_ID);
		String tableName = (String)request.get(MSG.REQUEST.TABLE_NAME);
		
		Object condition = request.get(MSG.REQUEST.CONDITION);
		Object values = request.get(MSG.REQUEST.UPDATE_VALUES);
		
		// UPDATE 메시지를 생성한다.
		Message msg = new Message();
		msg.set(MSG.REQUEST.USER_ID, userId);
		msg.set(MSG.REQUEST.TABLE_NAME, tableName);
		msg.set(MSG.ACTION, MSG.REQUEST.UPDATE);
		msg.set(MSG.REQUEST.UPDATE_VALUES, values);
		msg.set(MSG.REQUEST.CONDITION, condition);
		
		// 연결 요청용 MapReduce 를 생성해서, 요청을 뿌려준다.
		MapReduce mapReduce = new MapReduce(manager.getConnections());
		Message[] responses = null;
		try {
			responses = mapReduce.map(msg);
		} catch (InterruptedException e) {
			e.printStackTrace();
			response.setFailed(e.getMessage());
			return;
		}
 		
		// 성공적으로 지운 갯수
		int updatedCount = 0;
		
		// 결과를 종합한다.
		for (Message res : responses) {
			if (res.isSuccessed())
				updatedCount += (Integer)res.get(MSG.RESPONSE.COUNT);
			else
				System.err.println("update error: " + res.getErrorMessage());
		}
		
		// 결과 메시지를 만들어준다. 업데이트 된 개수를 담아준다.
		response.setSuccessed();
		response.set(MSG.RESPONSE.COUNT, updatedCount);
	}

}
