package contrabase.monitor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;

public class GraphListener implements ActionListener {
	private String action;

	private MonitorMainFrame monitorMainFrame;
	private JPanel repositoryTabPanel;

	private CategoryAxis xAxis;
	private ValueAxis yAxis;
	private CategoryPlot plot;
	private DefaultCategoryDataset dataset;
	private BarRenderer render;
	private JFreeChart chart;
	private String[] row;
	private String[] column;
	private List<String> repositoryList;
	private HashMap<String, String> repositoryMap;
	
	public GraphListener(MonitorMainFrame monitorMainFrame, JPanel repositoryTabPanel, String action) {
		this.monitorMainFrame = monitorMainFrame;
		this.action = action;
		this.repositoryTabPanel = repositoryTabPanel;
		setRepositoryList();
		
		row = new String[repositoryMap.size()];
		dataset = new DefaultCategoryDataset();
		for (int i = 0; i < repositoryMap.size(); i++) {
			row[i] = "repository" + (i + 1);
		}
	}
	
	public void actionPerformed(ActionEvent arg0) {
		column = new String[1];
		column[0] = action;
		
		for (int i = 0; i < row.length; i++) {
			for (int j = 0; j < column.length; j++) {
				String rid = repositoryMap.get(row[i]);
				Map<String, Long> result = monitorMainFrame.getDataByAction(action);
				long data = result.get(rid); 
				dataset.addValue(data, row[i], column[j]);
			}
		}

		render = new BarRenderer();
		xAxis = new CategoryAxis("RepositoryList");
		yAxis = new NumberAxis(action);
		plot = new CategoryPlot(dataset, xAxis, yAxis, render);
		chart = new JFreeChart(plot);
		
		ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setBounds(1, 1, 600, 360);
		JScrollPane repositoryPane = new JScrollPane();
		repositoryPane.setLayout(null);
		repositoryPane.add(chartPanel);
		repositoryPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		repositoryPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		if (repositoryTabPanel.getComponentCount() == 3)
			repositoryTabPanel.remove(2);
		repositoryTabPanel.add(repositoryPane);

		repositoryPane.setBounds(0, 45, 604, 411);
		monitorMainFrame.repaint();		
	}
	
	private void setRepositoryList(){
		repositoryList = monitorMainFrame.getRepositoryList();
		repositoryMap = new HashMap<String, String>();
		int repositoryNo = 1;
		
		Iterator<String> it = repositoryList.iterator();
		
		while(it.hasNext()){
			String repositoryName = it.next();
			repositoryMap.put("repository" + repositoryNo, repositoryName);
			repositoryNo++;
		}
	}
}

