package contrabase.monitor;

import java.awt.event.ContainerAdapter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.WindowConstants;
import javax.swing.SwingUtilities;

import contrabase.ContrabaseException;
import contrabase.api.ContrabaseConnector;
import contrabase.api.Table;
import contrabase.network.ContrabaseNetworkException;

public class MonitorMainFrame extends javax.swing.JFrame {
	private static final long serialVersionUID = 1L;
	private JPanel currentPanel;
	private ContrabaseConnector conn;

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MonitorMainFrame inst = new MonitorMainFrame();
				inst.setLocationRelativeTo(null);
				inst.setVisible(true);
			}
		});
	}
	
	public MonitorMainFrame() {
		super();
		initGUI();
	}
	
	private void initGUI() {
		try {
			this.setFocusCycleRoot(false);
			this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
			this.setTitle("Contrabase Monitor");
			setPanel(new LoginPanel(this), 270, 300);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void setPanel(JPanel newPanel, int width, int height) {
		if (currentPanel != null)
			getContentPane().remove(currentPanel);
		
		currentPanel = newPanel;
		getContentPane().add(currentPanel, "Center");
		currentPanel.setBounds(0, 0, width, height);
		currentPanel.setLayout(null);
		currentPanel.addContainerListener(new ContainerAdapter() {
		});
		setSize(currentPanel.getWidth(), currentPanel.getHeight());
	}
	
	public boolean login(String ip, int port, String id, String password) {
		try {
			conn = new ContrabaseConnector(ip, port, id, password);
			
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
			return false;
		}
		
		return true;
	}
	
	public void logout() {
		try {
			conn.close();
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
		}
	}
	
	public List<String> getTableList() {
		List<String> tableList = new ArrayList<String>();
		
		try {
			tableList = conn.getTableList();
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
			return tableList;
		}

		return tableList;
	}
	
	public Map<String, Long> getDataByAction(String action) {
		Map<String, Long> result = null;

		try {
			result = conn.getDataByAction(action);
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
		}
		
		return result;
	}
	
	public Table getTable(String tableName) {
		Table table = null;
		
		try {
			table = conn.getTable(tableName);
		} catch (ContrabaseNetworkException e) {
			e.printStackTrace();
		}
		
		return table;
	}

	public List<String> getRepositoryList(){
		List<String> repositoryList = new ArrayList<String>();
		
		try{
			repositoryList = conn.getRepositoryList();
		}catch(ContrabaseNetworkException e){
			e.printStackTrace();
			return repositoryList;
		}
		return repositoryList;
	}
}
