package contrabase.monitor;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import org.jfree.data.category.DefaultCategoryDataset;

import contrabase.ContrabaseException;
import contrabase.ObjectContainer;
import contrabase.ObjectContainerField;
import contrabase.api.ContrabaseResults;
import contrabase.api.Table;

public class SelectTableListener implements ActionListener {
	private MonitorMainFrame monitorMainFrame;
	private String tableName;
	private JPanel tableTabPanel;
	private List<String> repositoryList;
	private HashMap<String, String> repositoryMap;
	
	public SelectTableListener(MonitorMainFrame monitorMainFrame, JPanel tableTabPanel, String tableName) {
		this.monitorMainFrame = monitorMainFrame;
		this.tableName = tableName;
		this.tableTabPanel = tableTabPanel;
		setRepositoryList();
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
	    List<String[]> listRow = new ArrayList<String[]>();
	    String temp=null;
		try {
			Table table = monitorMainFrame.getTable(tableName);
			ContrabaseResults results = table.selectTableContent();

			
			while (results.hasNext()) {
				ObjectContainer objectContainer = results.nextObjectContainer();
				String row[] = new String[3];
				row[0] = objectContainer.getKey(); 
				row[1]= repositoryMap.get(objectContainer.getRepositoryId()); 
				
				StringBuffer value = new StringBuffer(objectContainer.getClassName()).append("\n");
				Iterator<ObjectContainerField> it = objectContainer.iterator();
				while (it.hasNext()) {
					ObjectContainerField field = it.next();
					value.append(field.getTypeName()).append(" : ")
						 .append(field.getFieldName()).append(" : ")
						 .append(field.getValue()).append("\n");
				}
				row[2] = value.toString();
				listRow.add(row);
			}
		} catch (ContrabaseException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		
	    DefaultTableModel tableModel = new DefaultTableModel() {
			private static final long serialVersionUID = 1L;

			public Class<String> getColumnClass(int columnIndex) {
	    		return String.class;
	    	}
	    };
	    
	    int size = listRow.size();
	    String[][] rowData = new String[size][3];
	    for (int i = 0; i < size; i++) {
	    	rowData[i] = listRow.get(i);
	    }
	    
	    String[] columnNames = {"Key", "RepositoryNo", "Value"};
	    tableModel.setDataVector(rowData, columnNames);
		JTable selectedTable = new JTable(tableModel);
	    int lines = 2;
	    selectedTable.setRowHeight(selectedTable.getRowHeight() * lines);
		selectedTable.setDefaultRenderer(String.class, new MultiLineCellRenderer());

		selectedTable.setShowGrid(true);

		JScrollPane tablePane = new JScrollPane(selectedTable);
		tablePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		tablePane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

		if (tableTabPanel.getComponentCount() == 2)
			tableTabPanel.remove(1);
		tableTabPanel.add(tablePane);
		tablePane.setBounds(131, 0, 473, 458);
		monitorMainFrame.repaint();
	}
	
	class MultiLineCellRenderer extends JTextArea implements TableCellRenderer {
		private static final long serialVersionUID = 1L;

		public MultiLineCellRenderer() {
			setLineWrap(true);
			setWrapStyleWord(true);
			setOpaque(true);
		}

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			if (isSelected) {
				setForeground(table.getSelectionForeground());
				setBackground(table.getSelectionBackground());
			} else {
				setForeground(table.getForeground());
				setBackground(table.getBackground());
			}
			setFont(table.getFont());
			if (hasFocus) {
				setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));
				if (table.isCellEditable(row, column)) {
					setForeground(UIManager
							.getColor("Table.focusCellForeground"));
					setBackground(UIManager
							.getColor("Table.focusCellBackground"));
				}
			} else {
				setBorder(new EmptyBorder(1, 2, 1, 2));
			}
			setText((value == null) ? "" : value.toString());
			return this;
		}
	}
	private void setRepositoryList(){
		repositoryList = monitorMainFrame.getRepositoryList();
		repositoryMap = new HashMap<String, String>();
		int repositoryNo = 1;
			
		Iterator<String> it = repositoryList.iterator();
		
		while(it.hasNext()){
			String repositoryName = it.next();
			repositoryMap.put(repositoryName,""+ repositoryNo);
			repositoryNo++;
		}
	}
}
