package contrabase.network;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;

import contrabase.network.ContrabaseNetworkException;

public class ClientSide {
	private Socket socket;
	private ObjectInputStream objectInputStream;
	private ObjectOutputStream objectOutputStream;
	
	/**
	 * 클라이언트에서 사용되는 소켓 wrapper 클래스
	 * 
	 * @param ip 접속할 서버의 아이피
	 * @param port 접속할 서버의 포트 번호
	 * @throws ContrabaseNetworkException
	 */
	public ClientSide(String ip, int port) throws ContrabaseNetworkException {
		try {
			this.socket = new Socket(ip, port);
			this.objectOutputStream = new ObjectOutputStream(this.socket.getOutputStream());
			this.objectInputStream = new ObjectInputStream(this.socket.getInputStream());
		} catch (UnknownHostException e) {
			throw new ContrabaseNetworkException("could't find the host.", e);
		} catch (IOException e) {
			throw new ContrabaseNetworkException("socket open error", e);
		}
	}
	
	/**
	 * 메시지를 보낸 후에, 응답을 받는다.
	 * XXX: 비동기로 응답을 받는 것으로 개선할 수 있을까? 쓰레드를 점유하지 않도록.
	 * 그러려면, 콜백이 있어야 하겠지. Runnable 을 하나 넣을 수 있어야 할테고. ㅋ
	 * 
	 * @param message
	 * @return 받은 메시지를 리턴한다.
	 * @throws IOException
	 * @throws ContrabaseNetworkException 
	 */
	public Message send(Message message) throws ContrabaseNetworkException {
		Messenger.send(objectOutputStream, message);
		return Messenger.receive(objectInputStream);
	}

	/**
	 * 연결을 끊어준다.
	 * 
	 * @throws ContrabaseNetworkException
	 */
	public void close() throws ContrabaseNetworkException {
		try {
			objectOutputStream.close();
			objectInputStream.close();
			socket.close();
		} catch (IOException e) {
			throw new ContrabaseNetworkException(e);
		}
	}
	
	/**
	 * 현재 접속의 로컬 아이피를 가져온다.
	 * 
	 * @return
	 */
	public String getLocalIP() {
		return socket.getLocalAddress().getHostAddress();
	}
	
	/**
	 * 현재 접속의 상대방의 아이피를 가져온다.
	 * 
	 * @return
	 */
	public String getRemoteIP() {
		return socket.getInetAddress().getHostAddress();
	}
}
