package contrabase.network;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

/**
 * @author Jong10
 * ServerSideThread 에서 사용하는, 요청 받을때마다 생성되는 핸들러용 쓰레드.
 * 일반적인 처리는 MessageHandler 를 이용해서 처리하면 된다.
 */
class HandlerThread extends Thread {
	private Socket socket;
	private MessageHandler messageHandler;
	
	public HandlerThread(Socket socket, MessageHandler messageHandler) throws IOException {
		this.socket = socket;
		this.messageHandler = messageHandler;
	}
	
	@Override
	public void run() {
		ObjectInputStream ois = null;
		ObjectOutputStream oos = null;
		
		try {
			oos = new ObjectOutputStream(socket.getOutputStream());
			ois = new ObjectInputStream(socket.getInputStream());
			
			Message request = null;
			Message response = null;
			
			// 받은 메시지를, 메시지 핸들러를 이용해서 처리한 후에,
			// 리턴된 응답을 보내준다. 클라이언트가 접속을 끊을때 까지.
			while ((request = (Message)ois.readObject()) != null) {
				response = new Message();
				messageHandler.handle(request, response);
				
				oos.writeObject(response);
				oos.flush();
			}
		} catch (EOFException e) {
			// close connection.
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			throw new RuntimeException("Never happened.");
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			// 사용한 스트림을 닫아준다.
			try {
				if (oos != null)
					oos.close();
				if (ois != null)
					ois.close();
			} catch(IOException e) {
				System.err.println("ERROR TYPE 2");
				e.printStackTrace();
			}
		}
	}
}
