package contrabase.network;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class Message implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private Map<String, Serializable> parameters;
	
	public Message() {
		parameters = new HashMap<String, Serializable>();
	}
	
	public void set(String key, Object value) {
		parameters.put(key, (Serializable)value);
	}
	
	public Object get(String key) {
		return parameters.get(key);
	}

	/**
	 * 자기 자신을 그대로 Deep-Copy 한 Message 객체를 리턴한다.
	 * @return 새로 만들어진 Message 객체
	 */
	public Message cloneMessage() {
		Message newMessage = new Message();
		for (Entry<String, Serializable> entry : parameters.entrySet())
			newMessage.set(entry.getKey(), entry.getValue());
		return newMessage;
	}
	
	
	// 편하게 하려고 만들어놓은 것들..
	
	public void setFailed(String errorMessage) {
		set(MSG.RESPONSE.RESULT, MSG.RESPONSE.FAILED);
		set(MSG.RESPONSE.ERROR_MESSAGE, errorMessage);
	}
	
	public void setSuccessed() {
		set(MSG.RESPONSE.RESULT, MSG.RESPONSE.SUCCESSED);
	}
	
	public boolean isSuccessed() {
		String result = (String) get(MSG.RESPONSE.RESULT);
		return result != null && result.equals(MSG.RESPONSE.SUCCESSED);
	}

	public boolean isFailed() {
		String result = (String) get(MSG.RESPONSE.RESULT);
		return result != null && result.equals(MSG.RESPONSE.FAILED);
	}
	
	public String getErrorMessage() {
		String errorMessage = (String) get(MSG.RESPONSE.ERROR_MESSAGE);
		return errorMessage == null ? "" : errorMessage;
	}
}
