package contrabase.network;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import contrabase.network.ContrabaseNetworkException;

public class Messenger {
	/**
	 * Message 객체를 해당 스트림으로 보낸다.
	 * 
	 * @param objectOutputStream
	 * @param message
	 * @throws ContrabaseNetworkException
	 */
	public static void send(ObjectOutputStream objectOutputStream, Message message) throws ContrabaseNetworkException {
		try {
			objectOutputStream.writeObject(message);
			objectOutputStream.flush();
		} catch (IOException e) {
			throw new ContrabaseNetworkException("Network Error at send()", e);
		}
	}

	/**
	 * Message 객체를 받는 static 메소드
	 * 
	 * @return
	 * @throws ContrabaseNetworkException 
	 */
	public static Message receive(ObjectInputStream objectInputStream) throws ContrabaseNetworkException {
		Message msg = null;
		
		try {
			msg = (Message)objectInputStream.readObject();
		} catch (ClassNotFoundException e) {
			e.printStackTrace(); // Message 와 같이 배포되는 한, 절대 발생할리 없음.
			throw new RuntimeException("Never happened.");
		} catch (EOFException e) {
			// do nothing. 연결 종료.
			System.err.println("Network is closed.");
		} catch (IOException e) {
			//System.err.println("Network Error at receive()");
			//e.printStackTrace();
			throw new ContrabaseNetworkException("Network Error at receive()", e);
		}
		
		return msg;
	}
}
