package contrabase.network;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import contrabase.network.ContrabaseNetworkException;

/**
 * Message 를 이용한 통신에서, 서버쪽을 담당하는 클래스.
 * @author Jong10
 */
public class ServerSideThread extends Thread {
	private ServerSocket serverSocket;
	private MessageHandler messageHandler;
	
	/**
	 * 생성자
	 * 
	 * @param port 오픈할 포트 번호
	 * @param messageHandler 메시지 핸들러
	 * @throws IOException
	 * @throws ContrabaseNetworkException 
	 */
	public ServerSideThread(int port, MessageHandler messageHandler) throws ContrabaseNetworkException {
		try {
			this.serverSocket = new ServerSocket(port);
		}
		catch (IOException e) {
			throw new ContrabaseNetworkException("Port " + Integer.toString(port) + " is busy..", e);
		}
		
		this.messageHandler = messageHandler;
	}
	
	@Override
	public void run() {
		while (true) {
			try {
				synchronized (serverSocket) {
					// 새로운 접속 요청이 들어오면,
					Socket clientSocket = serverSocket.accept();
					
					// 핸들러 쓰레드 생성 및 시작.
					HandlerThread handlerThread = new HandlerThread(clientSocket, messageHandler);
					handlerThread.start();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		} // end of while (true)
	}
	
	@SuppressWarnings("deprecation")
	public void close() throws ContrabaseNetworkException {
		try {
			// XXX : 쓰레드 멈추는 방법은 나중에 생각하자
			// 그냥 boolean 변수로는 되지 않는다. 네트워크 패킷을 기다리는 동안은 블럭 되는데,
			// 그 때에, flag를 설정 하더라도, 블럭에서 벗어나질 않는다.
			super.stop();
			
			serverSocket.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new ContrabaseNetworkException("ServerSocket Close Error", e);
		}
	}
}
