package contrabase.repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import contrabase.ObjectContainer;
import contrabase.network.MSG;
import contrabase.network.Message;
import contrabase.network.MessageHandler;
import contrabase.repository.datastore.ContrabaseDatastoreException;
import contrabase.repository.datastore.sqlite.SelectIterator;
import contrabase.util.Pair;

class DataHandler implements MessageHandler {
	private Datastore datastore;
	private int repositoryNo;
	private String repositoryId;

	public DataHandler(Datastore datastore) {
		this.datastore = datastore;
	}
	
	public void setRepositoryNo(int repositoryNo) {
		this.repositoryNo = repositoryNo;
	}
	public void setRepositoryId(String repositoryId){
		this.repositoryId = repositoryId;
	}

	/* (non-Javadoc)
	 * @see contrabase.network.MessageHandler#handle(contrabase.network.Message, contrabase.network.Message)
	 * 
	 * 데이터 핸들러 (select, load_balance 두 가지 경우만 처리)
	 */
	@Override
	public void handle(Message request, Message response) {
		String action = (String) request.get(MSG.ACTION);
		
		if (action.equals(MSG.API_TO_REPOSITORY.GET_DATA))
			handleGetData(request, response);
		else if (action.equals(MSG.MANAGEMENT.LOAD_BALANCE_ACTION))
			handleLoadBalanceData(request, response);
		else
			response.setFailed("invalid action");
	}

	private void handleGetData(Message request, Message response) {
		List<ObjectContainer> datalist = new ArrayList<ObjectContainer>();
		int reqCount = (Integer) request.get(MSG.API_TO_REPOSITORY.COUNT);
		String channelNo = (String) request.get(MSG.API_TO_REPOSITORY.CHANNEL);
		SelectIterator selectIterator = (SelectIterator) datastore.getChannel(channelNo);
		
		// 데이터를 들고와서, repositoryNo 를 설정한 후에, 갯수만큼 담아서 client 로 날린다.
		for (int count = 0; count < reqCount; count++) {
			if (selectIterator.hasNext()) {
				ObjectContainer newObject = selectIterator.next();
				newObject.setRepositoryNo(repositoryNo);
				newObject.setRepositoryId(repositoryId);
				datalist.add(newObject);
			}
			else {
				break;
			}
		}
		response.set(MSG.REPOSITORY_TO_API.DATA, datalist);
		response.setSuccessed();
	}

	/**
	 * 데이터 목록을 받아서, 모두 추가한다.
	 * @param request
	 * @param response
	 */
	@SuppressWarnings("unchecked")
	private void handleLoadBalanceData(Message request, Message response) {
		String userId = (String)request.get(MSG.REQUEST.USER_ID);
		String tabletName = (String)request.get(MSG.REQUEST.TABLE_NAME);
		List<ObjectContainer> data = (List<ObjectContainer>)request.get(MSG.MANAGEMENT.LOAD_BALANCE_DATA);
		
		for (ObjectContainer value : data) {
			try {
				datastore.insert(userId, tabletName, value);
			} catch (ContrabaseDatastoreException e) {
				e.printStackTrace();
				response.setFailed(e.getLocalizedMessage());
				return;
			}
		}
		
		response.setSuccessed();
	}
}
