package contrabase.repository;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Calendar;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import org.apache.log4j.*;

import contrabase.master.MasterGUI;

public class RepositoryGUI extends JFrame {
	private static final long serialVersionUID = 1L;
	private JButton clear;
	private JTextArea logView;
	private JScrollPane logScroll;
	private JPanel mainPanel;
	private Logger logger;
	private String layout;
	private String logFileName;

	public RepositoryGUI(String logDir) {
		logFileName = logDir + "/repository.log";
	}
	
	public void start() {
		clear = new JButton("Clear");
		logView = new JTextArea();
		logScroll = new JScrollPane(logView);
		mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		logView.setLineWrap(true);
		logView.setEditable(false);
		logScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		logScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		mainPanel.add(BorderLayout.CENTER, logScroll);
		mainPanel.add(BorderLayout.SOUTH, clear);
		
		clear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				logView.setText("");
			}
		});
		
		this.add(mainPanel);
		this.setSize(650, 500);
		this.setTitle("Repository");
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// log4j 관련 부분
		logger = Logger.getLogger(MasterGUI.class);
		layout = "[%d{yyyy-MM-dd HH:mm}] %m %n";
		PatternLayout patternLayout = new PatternLayout(layout);
		DailyRollingFileAppender appender = null;

		try {
			String datePattern = ".yyyy-MM-dd-HH";
			appender = new DailyRollingFileAppender(patternLayout, logFileName, datePattern);
		} catch (IOException e) {
			e.printStackTrace();
		}
		logger.addAppender(appender);
		logger.setLevel(Level.INFO);
	}

	public void appendLog(String log) {
		Calendar cal = Calendar.getInstance();
		String s = String.format("[%04d-%02d-%02d %02d:%02d]", cal.get(Calendar.YEAR), (cal.get(Calendar.MONTH) + 1), 
				cal.get(Calendar.DAY_OF_MONTH),cal.get(Calendar.HOUR_OF_DAY), cal.get(Calendar.MINUTE));
		logger.info(log);
		logView.append(s + " " + log + "\n");
		logScroll.getVerticalScrollBar().setValue(logScroll.getVerticalScrollBar().getMaximum());
	}
}
