package contrabase.repository;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.*;

import contrabase.util.FileUtil;
/**
 *   RepositoryID를 생성하는 클래스 
 *   Repository 가 접속했을때의 millisecond 와 Repository 의 IP Port 를 조합하여 ID를 만드는 클래스
 *  기본 동작은 rid.txt 파일이 있으면 그 파일에서 rid를 읽어오고 없으면 rid를 위의 조합으로 생성한다.
 * */
public class RepositoryID {

	public static String getRid(String dataDir, int port) throws IOException, NoSuchAlgorithmException {
		String rid = null;
		FileUtil.createDirectory(dataDir);
		File file = new File(dataDir + File.separator + "rid.txt");
		
		if (file.exists())
			rid = readRid(file);
		else
			rid = generateNewRid(dataDir, port);
		
		return rid;
	}

	private static String readRid(File file) throws FileNotFoundException, IOException {
		String rid;
		FileReader fr = null;
		BufferedReader br = null;

		fr = new FileReader(file);
		br = new BufferedReader(fr);

		rid = br.readLine();

		fr.close();
		br.close();
		return rid;
	}

	private static String generateNewRid(String dataDir, int port) throws IOException, UnknownHostException {
		String rid = InetAddress.getLocalHost().getHostAddress() + "/" + port + "/" + Long.valueOf(System.currentTimeMillis());
		
		FileWriter fw = new FileWriter(dataDir + File.separator + "rid.txt");
		fw.write(rid);
		fw.close();
		return rid;
	}
}
