package contrabase.repository.datastore.sqlite;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.SortedSet;
import java.util.TreeSet;

import contrabase.condition.KeySelector;
import contrabase.condition.Term;
import contrabase.condition.Term.TYPE;

public class SQLiteKeySelector implements KeySelector {
	
	private Statement statement;
	
	/**
	 * 생성자
	 * @param statement DB 접속 커넥션
	 */
	public SQLiteKeySelector(Statement statement) {
		this.statement = statement;
	}

	/**
	 * TYPE 이름으로, SQLite 에서 사용하는 테이블 이름으로 변환해준다.
	 * TYPE.toString()을 해도 되긴 하지만,
	 * TYPE 이름과 테이블 이름이 일치하지 않는 경우를 위해서 일일이 검사한다.
	 * 
	 * @param type
	 * @return 해당 타입에 대한 테이블 이름을 돌려준다.
	 */
	private String getTableNameFromType(TYPE type) {
		if (TYPE.NUMBER.equals(type))
			return "TABLET_NUMBER";
		else if (TYPE.BOOLEAN.equals(type))
			return "TABLET_BOOLEAN";
		else if (TYPE.TEXT.equals(type))
			return "TABLET_TEXT";
		else
			throw new RuntimeException("Never Happened!! => BAD TYPE: " + type);
	}

	@Override
	public SortedSet<String> getKeysByTerm(Term term) {
		SortedSet<String> keys = new TreeSet<String>();
		String sql = null;
		String value = term.getValue();
		
		// <key> 인지 검사해서, 조건이 <key> == "*" 항목이면, DB에 있는지 확인하는 sql 문을 만든다.
		if (term.isAllKeyCompare()) {
			sql = "SELECT key FROM TABLET_KEYS ORDER BY key ASC";
		}
		
		// <key> 인지 검사해서, 조건이 <key> == ?? 항목이면, DB에 있는지 확인하는 sql 문을 만든다.
		else if (term.isKeyCompare()) {
			sql = "SELECT key FROM TABLET_KEYS WHERE key = '" + value + "'";
		}
		
		// 아니면, 해당 조건에 맞는 key를 찾는 sql 문을 만든다.
		else
			sql = getSqlByTermStatement(term, value);
		
		// 해당 sql 문으로, key가 있는지 구한다.
		try {
			ResultSet rs = statement.executeQuery(sql);
			
			// 검색된 key를 모두 Set에 추가한다.
			while (rs.next())
				keys.add(rs.getString("key"));
			
		} catch (SQLException e) {
			// 에러나면, 그냥, 해당하는 키가 없는거임.
			e.printStackTrace();
		}

		return keys;
	}

	private String getSqlByTermStatement(Term term, String termValue) {
		String value = null;
		if (term.getType().equals(TYPE.BOOLEAN))
			value = new Integer(SQLiteUtil.boolStringToInt(termValue)).toString();
		else
			value = term.getFormattedValue();
		
		String sql = new StringBuffer("SELECT key FROM ")
					.append(getTableNameFromType(term.getType()))
					.append(" WHERE ")
					.append("(var_name == '")
					.append(term.getField())
					.append("') AND (value ")
					.append(term.getCondition())
					.append(" ")
					.append(value)
					.append(")")
					.toString();
		return sql;
	}
}
