package contrabase.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public class Converter {
	/**
	 * 해당 객체를 바이트 배열로 변환한다.
	 * 
	 * @param obj 변환할 객체
	 * @return 변환된 바이트 배열
	 * @throws IOException
	 */
	public static byte[] objectToBytes(Serializable obj)
	throws IOException {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(bos);
		oos.writeObject(obj);
		oos.flush();
		oos.close();
		bos.close();
		
		return bos.toByteArray();
	}
	
	/**
	 * 바이트 배열을, 객체로 변환한다.
	 * 
	 * @param bytes 변환할 바이트 배열
	 * @return 변환된 객체
	 * @throws IOException 
	 * @throws ClassNotFoundException 
	 */
	public static Serializable bytesToObject(byte[] data)
	throws IOException, ClassNotFoundException {
		Serializable obj;
		ByteArrayInputStream bis = new ByteArrayInputStream(data);
		ObjectInputStream ois = new ObjectInputStream(bis);
		obj = (Serializable)ois.readObject();
		ois.close();
		bis.close();
		
		return obj;
	}
	
	public static Serializable doubleToObject(String typeName, Double d) {
		Serializable obj = null;
		
		if ("byte".equals(typeName) ||
				"java.lang.Byte".equals(typeName)) {
			obj = d.byteValue();
		}
		else if ("short".equals(typeName) ||
				"java.lang.Short".equals(typeName)) {
			obj = d.shortValue();
		}
		else if ("int".equals(typeName) || 
				"java.lang.Integer".equals(typeName)) {
			obj = d.intValue();
		}
		else if ("long".equals(typeName) ||
				"java.lang.Long".equals(typeName)) {
			obj = d.longValue();
		}
		else if ("float".equals(typeName) ||
				"java.lang.Float".equals(typeName)) {
			obj = d.floatValue();
		}
		else if ("double".equals(typeName) ||
				"java.lang.double".equals(typeName)) {
			obj = d.doubleValue();
		}
		
		return obj;
	}
}
